﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Oracle.DataAccess.Client;

namespace OccupationMonitorLib.DataAccess
{
    /// <summary>
    /// Trieda poskytuje mapovanie medzi id pocitacov v ucebniach a ich distiguish (domenovym) menom
    /// </summary>
    class ADMapper
    {
        /// <summary>
        /// metoda vrati distinguish name pocitaca, ktoreho id dostane ako argument
        /// </summary>
        /// <param name="pcId"></param>
        /// <returns></returns>
        internal string GetDistinguishName(long pcId)
        {
            string distName = String.Empty;

            using (OracleCommand cmd = new OracleCommand("SELECT dn FROM NAMTAR_WORKSTATIONS WHERE workstation_id=:id", ConnectionSingleton.Instance.Connection))
            {
                cmd.Parameters.Add("@id", pcId);

                using (OracleDataReader reader = cmd.ExecuteReader())
                {
                    if (reader.Read())
                    {
                        distName = Convert.ToString(reader["DN"]);
                    }
                }
            }
            return distName;
        }

        /// <summary>
        /// vrati vsetky dvojice pc_id-pc_distinguish_name
        /// </summary>
        /// <returns></returns>
        internal Dictionary<long, string> GetDistinguishNames()
        {
            Dictionary<long, string> names = new Dictionary<long, string>();
            string distName = String.Empty;
            long pcId = 0;

            using (OracleCommand cmd = new OracleCommand("SELECT workstation_id,dn FROM NAMTAR_WORKSTATIONS", ConnectionSingleton.Instance.Connection))
            {
                using (OracleDataReader reader = cmd.ExecuteReader())
                {
                    while (reader.Read())
                    {
                        distName = Convert.ToString(reader["DN"]);
                        pcId = Convert.ToInt64(reader["WORKSTATION_ID"]);
                        names.Add(pcId, distName);
                    }
                }
            }
            return names;
        }

    }

    
}
