﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Diagnostics;
using System.Text.RegularExpressions;

namespace OccupationMonitorLib.DataAccess
{
    public class ErrorLogger
    {
        private string _path;
        private string _fileName;
        private object _locker;
        private string _eventLogSource;

        public ErrorLogger(string p, string prefix, string logSource) 
        {
            _locker = new object();
            _path = p;
            _eventLogSource = logSource;

            Regex reg = new Regex("[^a-zA-Z0-9]");
            _fileName = reg.Replace(prefix, "");
        }

        public void LogException(Exception ex) 
        {   //logovanie chyby do suboru
            try
            {
                lock (_locker)
                {
                    using (StreamWriter writer = File.AppendText(String.Format("{0}{1}.txt", _path, _fileName))) 
                    {

                        writer.WriteLine("{0} Type of exception: {1} Message: {2} Source: {3} Stack Trace: {4}", 
                            DateTime.Now, ex.GetType() + Environment.NewLine, GetExceptionMessage(ex) + Environment.NewLine, ex.Source + Environment.NewLine, ex.StackTrace);
                    }
                }
            }
            //logovanie chyby do event viewer ak zlyha do suboru
            catch (Exception exToLog)
            {
                try
                {
                    if (!EventLog.SourceExists(_eventLogSource))
                    {
                        EventLog.CreateEventSource(_eventLogSource, "Application");
                        return;
                    }

                    using (EventLog log = new EventLog())
                    {
                        log.Source = _eventLogSource;
                        string originalEx = String.Format("Original exception: {0}{1}StackTrace: {2}", GetExceptionMessage(ex), Environment.NewLine, ex.StackTrace);
                        string tofileEx = String.Format("Exception thrown while logging to file: {0}{1}StackTrace: {2}", GetExceptionMessage(exToLog), Environment.NewLine, exToLog.StackTrace);
                        string toLog = String.Format("{0}{1}-----------{1}{2}", originalEx, Environment.NewLine, tofileEx);
                        log.WriteEntry(toLog, EventLogEntryType.Error);
                    }
                }
                catch (Exception) { }
            }
        }

        private StringBuilder GetExceptionMessage(Exception ex) 
        {
            StringBuilder builder = new StringBuilder();
            builder.Append(ex.Message);

            if (ex.InnerException != null)
            {
                return builder.Append(GetExceptionMessage(ex.InnerException));
            }
            else 
            {
                return builder;
            }
        }


    }
}
