﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using OccupationMonitorLib.MiniZappa;

namespace OccupationMonitorLib.DataAccess
{
    /// <summary>
    /// trieda sluzi na poskytovanie lock objectov od ciest k suborom (obrazkom), kazdy subor reprezentuje lokalitu
    /// </summary>
    static public class ImgsPaths
    {
        volatile static Dictionary<string, System.Object> _paths; //<cesta k obrazku, lock object>
        static string _imgDirectory; //cesta k zlozke

        public static void Initialize(string dir)
        {
            _paths = new Dictionary<string, System.Object>();
            _imgDirectory = dir;

            Actualize();
        }

        public static Object GetPath(string path)
        {
            if (_paths.ContainsKey(path)) return _paths[path]; //vrati lock object danej cesty len ak ho ma
            else
            {
                Actualize();//pokusi sa aktualizovat zoznam dostupnych ciest
                if (_paths.ContainsKey(path))
                {
                    return _paths[path];
                }
                else //minizappa nepozna lokalitu
                {
                    throw new ArgumentException(String.Format("Error while getting lock to path: {0}", path));
                }
            }
        }

        private static void Actualize()
        {
            using (MiniZappaNewClient mz = new MiniZappaNewClient())
            {
                MapLocality ml;
                string path;
                long[] aml = mz.GetAvailableMapLocalities();
                for (int i = 0; i < aml.Length; i++)
                {
                    ml = mz.GetMaplocality(aml[i]);
                    if (ml != null)
                    {
                        path = String.Format("{0}{1}.bmp", _imgDirectory, ml.Id);
                        if (!(_paths.ContainsKey(path)))
                        {
                            _paths.Add(path, new Object());
                        }
                    }
                }
            }
        }


    }
}
