﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Oracle.DataAccess.Client;
using OccupationMonitorLib.MiniZappa;
using OccupationMonitorLib.DataObjects;

namespace OccupationMonitorLib.DataAccess
{
    //trieda zodpovedna za komunikaciu s databazou
    internal class LogItemsCatalog
    {

        internal void Create(LogItem item)
        {
            using (OracleCommand cmd = new OracleCommand("INSERT INTO LOG_ITEMS VALUES(:rootId, :pcId, :time, :state)", ConnectionSingleton.Instance.Connection))
            {
                cmd.Parameters.Add("@rootId", item.RootId);
                cmd.Parameters.Add("@pcId", item.PcId);
                cmd.Parameters.Add("@time", item.Time);
                cmd.Parameters.Add("@state", (int)item.State);

                cmd.ExecuteScalar();
            }
        }

        //vrati posledny zaznam vztahujuci sa k urcitej stanici
        internal LogItem GetLastLog(long rootId, long pcId)
        {
            LogItem item = new LogItem();
            using (OracleCommand cmd = new OracleCommand("SELECT * FROM (SELECT* FROM LOG_ITEMS WHERE ROOT_ID = :root AND PC_ID= :pc ORDER BY TIME DESC) WHERE ROWNUM<=1", ConnectionSingleton.Instance.Connection))
            {
                cmd.Parameters.Add("@root", rootId);
                cmd.Parameters.Add("@pc", pcId);

                using (OracleDataReader reader = cmd.ExecuteReader())
                {
                    if (reader.Read()) item = ReadRow(reader);
                }
            }

            return item;
        }

        //vrati zaznamy pocitacov ulozene v casovom intervale - zoznam zaznamov je usporiadany podla id pocitacov a casu
        internal List<LogItem> GetItems(long rootId, DateTime from, DateTime to)
        {
            List<LogItem> items = new List<LogItem>();

            using (OracleCommand cmd = new OracleCommand("SELECT * FROM LOG_ITEMS WHERE ROOT_ID = :root AND TIME >= :timeFrom AND TIME<=:timeTo ORDER BY PC_ID, TIME", ConnectionSingleton.Instance.Connection))
            {
                cmd.Parameters.Add("@root", rootId);
                cmd.Parameters.Add("@timeFrom", from);
                cmd.Parameters.Add("@timeTo", to);

                using (OracleDataReader reader = cmd.ExecuteReader())
                {
                    while (reader.Read()) items.Add(ReadRow(reader));
                }
            }

            return items;
        }

        private static LogItem ReadRow(OracleDataReader reader)
        {
            LogItem item = new LogItem();
            string str = reader["ROOT_ID"].ToString();
            item.RootId = Convert.ToInt64(str);
            str = reader["PC_ID"].ToString();
            item.PcId = Convert.ToInt64(str);
            item.Time = (DateTime)reader["TIME"];
            item.State = (WorkstationStatusEnum)(Convert.ToInt32(reader["STATE"]));

            return item;
        }


    }
}
