﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using OccupationMonitorLib.MiniZappa;

namespace OccupationMonitorLib.Drawers
{
    //trieda zodpovedna za vykreslovanie obrazkov obsadenosti
    class OccupationDrawer
    {
        #region Private fields
        private Pen pen;
        private int borderLength;
        private int pcLength;
        private int penSize;
        private int legendHeight;
        #endregion

        public OccupationDrawer(int border, int pc, int pen, int legend) 
        {
            borderLength = border;
            pcLength = pc;
            penSize = pen;
            legendHeight = legend;
        }

        //vrati obrazok reprezentujuci obsadenost danej lokality
        internal Image DrawOccupation(MapLocality ml, Dictionary<long, WorkstationStatus> ws)
        {
            Bitmap bmp = new Bitmap((int)ml.Width, (int)ml.Height + legendHeight);

            using (Graphics g = Graphics.FromImage(bmp))
            {
                using (pen = new Pen(Color.Black, penSize))
                {
                    g.Clear(Color.White);
                    g.DrawRectangle(pen, 0, 0, bmp.Width - pen.Width, bmp.Height - pen.Width);
                    for (int i = 0; i < ml.Studyrooms.Length; i++)
                    {
                        MapObject room = ml.Studyrooms[i].RootMapObject;
                        g.DrawRectangle(pen, (int)room.X, (int)room.Y, (int)room.Width - pen.Width, (int)room.Height - pen.Width);
                        DrawChildren(ws, room, g, 0, 0);
                    }
                    DrawLegend(g, (int)ml.Height);
                    DrawTime(g, bmp.Width, bmp.Height);
                    return bmp;
                }
            }
        }

        //do obrazka zakresli cas, v ktorom sa obrazok generuje
        private void DrawTime(Graphics g, int width, int height) 
        {
            string toDraw = "Obsazenost v čase: " + DateTime.Now.ToString("dd.MM.yyyy hh:mm:ss");

            using (Font font = new Font("Times New Roman", 9.0f))
            {
                SizeF strSize = g.MeasureString(toDraw, font, width);

                PointF p = new PointF();
                p.X = width - strSize.Width;
                p.Y = height - 20;
                g.DrawString(toDraw, font, new SolidBrush(Color.LightSlateGray), p);
            }
        }

        //do obrazka zakresli legendu - v pripade zmien nutne zmenit aj farbu reprezentujucu dany stav v metode GetColor
        private void DrawLegend(Graphics g, int mlHeight) 
        {
            using (Font f = new Font("Times New Roman", 9.0f))
            {
                PointF p = new PointF();
                
                DrawPc(g, Color.Green, 10, mlHeight + 10);
                p.X = 10 + borderLength;
                p.Y = mlHeight + 10 + ((borderLength - pcLength) / 2) - 2;
                g.DrawString("PC Volný", f, new SolidBrush(Color.Black), p);

                DrawPc(g, Color.Red, 10, mlHeight + 10 + borderLength);
                p.X = 10 + borderLength;
                p.Y = mlHeight + 10 + borderLength + ((borderLength - pcLength) / 2) - 2;
                g.DrawString("PC Obsazený", f, new SolidBrush(Color.Black), p);

                DrawPc(g, Color.Blue, 10, mlHeight + 10 + 2 * borderLength);
                p.X = 10 + borderLength;
                p.Y = mlHeight + 10 + 2 * borderLength + ((borderLength - pcLength) / 2) - 2;
                g.DrawString("PC Vypnutý", f, new SolidBrush(Color.Black), p);

                DrawPc(g, Color.Black, 10, mlHeight + 10 + 3 * borderLength);
                p.X = 10 + borderLength;
                p.Y = mlHeight + 10 + 3 * borderLength + ((borderLength - pcLength) / 2) - 2;
                g.DrawString("Stav PC nedostupný", f, new SolidBrush(Color.Black), p);
            }
        }

        //vykresli vsetkych potomkov uzlu 'room'
        private void DrawChildren(Dictionary<long, WorkstationStatus> ws, MapObject room, Graphics g, int moveX, int moveY)
        {
            for (int i = 0; i < room.Children.Length; i++)
            {
                if (room.Children[i] != null)
                {
                    if (room.Children[i].Data != null)
                    {
                        if (room.Children[i].ObjectType == MapObjectType.Workstation)
                        {
                            Color color = Color.Black;
                            if (ws.ContainsKey((long)room.Children[i].Data))
                            {
                                color = GetColor(ws[(long)room.Children[i].Data].StatusValue);
                            }
                            DrawPc(g, color, (int)room.Children[i].X + (int)room.X + moveX, (int)room.Children[i].Y + (int)room.Y + moveY);
                        }
                        else if (room.Children[i].ObjectType == MapObjectType.Studyroom)
                        {
                            g.DrawRectangle(pen, (int)room.Children[i].X, (int)room.Children[i].Y, (int)room.Children[i].Width - pen.Width, (int)room.Children[i].Height - pen.Width);
                            DrawChildren(ws, room.Children[i], g, (int)room.X, (int)room.Y);
                        }
                        else if (room.Children[i].ObjectType == MapObjectType.Label)
                        {
                            using (Font font = new Font("Times New Roman", 12.0f))
                            {
                                PointF point = new PointF();
                                point.X = Convert.ToSingle(room.Children[i].X) + Convert.ToSingle(room.X);
                                point.Y = Convert.ToSingle(room.Children[i].Y) + Convert.ToSingle(room.Y);
                                string text = room.Children[i].Data.ToString();
                                g.DrawString(text, font, new SolidBrush(Color.Black), point);
                            }
                        }
                    }
                }
            }
        }

        //vykresli pocitac - reprezentovany listom stromu
        private void DrawPc(Graphics g, Color color, int x, int y)
        {
            g.FillRectangle(new SolidBrush(Color.White), x, y, borderLength, borderLength);
            
            int move = (borderLength - pcLength) / 2;
            g.FillRectangle(new SolidBrush(color), x + move, y + move, pcLength, pcLength);
            g.DrawRectangle(pen, x + move, y + move, pcLength, pcLength);
        }

        //vrati farbu, ktora reprezentuje stav predany v parametri - v pripade zmien nutne zmenit aj legendu
        private Color GetColor(WorkstationStatusEnum status)
        {
            if (status == WorkstationStatusEnum.Free) return Color.Green;
            else if (status == WorkstationStatusEnum.Occupied) return Color.Red;
            else if (status == WorkstationStatusEnum.Off) return Color.Blue;
            else return Color.LightGray;
        }

        
    }
}
