﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using OccupationMonitorLib.DrawingStrategies;
//using OccupationMonitor.DataAccess;

namespace OccupationMonitorLib.Drawers
{
    //trieda zodpovedna za vykreslenie statistik
    class StatsDrawer
    {
        #region Private fields
        private Pen pen;
        private int statsPadding;
        private DrawXAxisStrategy xStrat;
        private float partLength;
        private float coef;
        private int penSize;
        #endregion

        public StatsDrawer(DrawXAxisStrategy strategy, int padding, int part, float cf, int penS) 
        {
            statsPadding = padding;
            xStrat = strategy;
            partLength = part;
            coef = cf;
            penSize = penS;
        }

        //vrati obrazok statistik
        internal Image DrawStatistics( float[] intervals, int width, int height, DateTime from, DateTime to) 
        {
            Bitmap bmp = new Bitmap(width, height);

            using (pen = new Pen(Color.Black, penSize))
            {
                using (Graphics g = Graphics.FromImage(bmp))
                {
                    g.FillRectangle(new SolidBrush(Color.White), 0, 0, width, height);
                    g.FillRectangle(new SolidBrush(Color.LightBlue), statsPadding, statsPadding, width - statsPadding - statsPadding, height - statsPadding - statsPadding);
                    xStrat.DrawXAxis(g, pen, bmp.Width - statsPadding, bmp.Height - statsPadding + pen.Width, intervals.Length, statsPadding, partLength);

                    DrawYAxis(g, bmp.Width - statsPadding + pen.Width, bmp.Height - statsPadding + pen.Width, intervals.Length);
                    DrawStats(g, bmp.Width - 2 * statsPadding, bmp.Height - 2 * statsPadding, intervals);
                    DrawInterval(g, width, from, to);
                }
            }
            return bmp;        
        }

        //do obrazka zapise interval, v ktorom sa zbierali data potrebne na vygenerovanie statistiky
        private void DrawInterval(Graphics g, int width, DateTime dateFrom, DateTime dateTo) 
        {
            string toDraw = "Statistika vychází z dat získaných v intervalu " + dateFrom.ToString("dd.MM.yyyy") + " - " + dateTo.ToString("dd.MM.yyyy") + ". Statistika může obsahovat zkreslení.";
            int maxWidth = width;
            
            using (Font font = new Font("Times New Roman", 9.0f))
            {
                SizeF strSize = g.MeasureString(toDraw, font, maxWidth);

                PointF p = new PointF();
                p.X = width - strSize.Width - statsPadding;
                p.Y = 10;
                g.DrawString(toDraw, font, new SolidBrush(Color.LightSlateGray), p);
            }
        }

        //vykresli y-ovu os
        private void DrawYAxis(Graphics g, float width, float height, int xCount) 
        {
            g.DrawLine(pen, statsPadding, height - pen.Width, statsPadding, statsPadding);
            
            float part = (height - statsPadding) / 10;
            for (int i = 0; i < 10; i++)
            {
                pen.Color = Color.LightSlateGray;
                g.DrawLine(pen, statsPadding, i * part + statsPadding, width-pen.Width , i * part + statsPadding);
                pen.Color = Color.Black;
                Draw(g, (100 - i * 10).ToString() + "%", statsPadding - 30, i * part + statsPadding);
            }
        }

        //vykresli graf
        private void DrawStats(Graphics g, float widht, float height, float[] intervs) 
        {
            float partWidht = (widht)/intervs.Length;
            for (int i = 0; i < intervs.Length; i++) 
            {
                DrawStatsPart(g, i, partWidht, height, intervs[i]);
            }
        }

        //vykresli cast grafu = 1 stlpec v grafe
        private void DrawStatsPart(Graphics g, float xPos, float partWidht, float maxHeight, float value) 
        {
            float leftX = Convert.ToInt32(xPos*partWidht + (coef)*partWidht);
            float rightX = Convert.ToInt32(xPos * partWidht + (1-coef) * partWidht);
            
            float upY = Convert.ToInt32(maxHeight - maxHeight*(value*0.01) + statsPadding);// deli stovkou -> prevod na percenta

            float width = rightX - leftX;
            float height = maxHeight - upY+statsPadding;

            g.FillRectangle(new SolidBrush(Color.Green), leftX+statsPadding, upY, width, height);
            g.DrawRectangle(pen, leftX+statsPadding, upY, width, height);
        }

        //vykresli hodnoty k y-ovej osy (percenta)
        private void Draw(Graphics g, string text, float x, float y) 
        {
            using (Font font = new Font("Times New Roman", 9.0f))
            {
                PointF point = new PointF();
                point.X = x;
                point.Y = y;
                g.DrawString(text, font, new SolidBrush(Color.Black), point);
            }
        }



    }
}
