﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using OccupationMonitorLib.Enums;
using System.ServiceModel;
using OccupationMonitorLib.MiniZappa;

namespace OccupationMonitorLib
{
    [ServiceContract]
    public interface IOccupationMonitor
    {
        ///<param name="roomId">Studyroom id.</param>
        ///<returns>The image representing occupation of the studyroom.</returns>
        ///<exception cref="FaultException"> Is thrown when an error occurs, usually when room with id <paramref name="roomId"> does not exist.</exception>
        [OperationContract]
        byte[] GetImagesFromFile(long roomId);

        ///<param name="roomId">Studyroom id.</param>
        ///<returns>The statistics of utilization rate of the studyroom.</returns>
        ///<exception cref="FaultException"> Is thrown when an error occurs.</exception>
        [OperationContract]
        byte[] GetStatsImg(long roomId, StatFormat format);

        ///<param name="roomId">Studyroom id.</param>
        ///<returns>The number of free computers in specified studyroom.</returns>
        ///<exception cref="FaultException"> Is thrown when an error occurs, usually when room with id <paramref name="roomId"> does not exist</exception>
        [OperationContract]
        int GetFreePcsCount(long roomId);

        ///<param name="roomId">Studyroom id.</param>
        ///<returns>The number of computers in specified studyroom.</returns>
        ///<exception cref="FaultException"> Is thrown when an error occurs, usually when room with id <paramref name="roomId"> does not exist.</exception>
        [OperationContract]
        int GetPcsCount(long roomId);

        ///<returns>IDs and names of all available localities.</returns>
        ///<exception cref="FaultException"> Is thrown when an error occurs.</exception>
        [OperationContract]
        Dictionary<long, string> GetAvailableLocalities();

        void LogState(MapLocality locality, Dictionary<long, WorkstationStatus> statuses);

        void GenerateImage(MapLocality ml, Dictionary<long, WorkstationStatus> ws);

        void GenerateCsv(MapLocality ml, Dictionary<long, WorkstationStatus> ws);

    }
}
