﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using OccupationMonitorLib.DataObjects;

namespace OccupationMonitorLib.ProcessingStrategies
{
    class ProcessToHours : ProcessingStrategy
    {
        internal override int MaxSeconds
        {
            get { return 111600; }//31*60*60
        }
        //index prvku vo vratenom poli reprezentuje hodinu pocas dna
        internal override int[] Process(List<OccupationInterval> intervals) 
        {
            int[] processed = new int[24];
            foreach (OccupationInterval interval in intervals)
            {
                if (ToSeconds(interval.Logout - interval.Login) < MaxLoginTime)//dlzka prihlasenia nesmie prekrocit 24 hodin
                {
                    if (interval.Login.Hour == interval.Logout.Hour && interval.Login.Day == interval.Logout.Day)//prihlasenie a odhlasenie v ramci 1 hodiny
                    {
                        processed[interval.Login.Hour] += ToSeconds(interval.Logout - interval.Login);
                    }
                    else 
                    {
                        //celkova dlzka prihlasenia
                        TimeSpan difference = interval.Logout - interval.Login;
                        
                        //cas ktory sa pripocita k hodine, kedy sa uzivatel prihlasil
                        TimeSpan loginTimeInterval = GetDateToSubtract(interval) - interval.Login;
                        //cas ktory sa pripocita k hodine, kedy sa uzivatel odhlasil
                        TimeSpan logoutTimeInterval = interval.Logout - new DateTime(interval.Logout.Year, interval.Logout.Month, interval.Logout.Day, interval.Logout.Hour, 0, 0);
                        TimeSpan diff = loginTimeInterval + logoutTimeInterval;
                        difference -= diff;

                        processed[interval.Login.Hour] += ToSeconds(loginTimeInterval);//pripocita cas k hodine prihlasenia
                        processed[interval.Logout.Hour] += ToSeconds(logoutTimeInterval);//pripocita cas k hodine odhlasenia

                        //v cykle sa pripocita cas k vsetkym hodinam, ktore su medzi hodinou prihlasenia a hodinou odhlasenia
                        if ((interval.Logout.Hour - interval.Login.Hour + 24) % 24 > 1 
                          ||((interval.Logout.Hour - interval.Login.Hour + 24) % 24 == 0 && ((interval.Login.Day+1)%24) == interval.Logout.Day))
                        {
                            for (int i = 0; i < difference.Hours; i++)
                            {
                                processed[(interval.Login.Hour + 1 + i) % 24] += 3600;
                            }
                        }
                    }
                }
            }
            return processed;
        }

        //ziska datum a cas nasledujucej celej hodiny nasledujucej po case prihlasenia
        private DateTime GetDateToSubtract(OccupationInterval interval) 
        {
            //prihlasenie prebehlo pocas 23. hodiny posledneho dna v mesiaci - nutne ziskat datum nasledujuceho dna
            if (interval.Login.Day == DateTime.DaysInMonth(interval.Login.Year, interval.Login.Month) && interval.Login.Hour == 23)
            {
                //prihlasenie prebehlo pocas poslednej hodiny v roku - takmer nulova pravdepodobnost ze to nastane
                if (interval.Login.Month == 12) return new DateTime(interval.Login.Year + 1, 1, 1, 0, 0, 0);
                else//prihlasenie prebehlo pocas poslednej hodiny v mesiaci
                    return new DateTime(interval.Login.Year, interval.Login.Month + 1, 1, 0, 0, 0);
            }
            else 
            if(interval.Login.Hour != 23)
                //prihlasenie prebehlo inokedy ako pocas poslednej hodiny dna, vrati ten isty datum, hodinu prihlasenia + 1, minuty a sekundy vynuluje
                return new DateTime(interval.Login.Year, interval.Login.Month, interval.Login.Day, (interval.Login.Hour + 1) % 24, 0, 0);
            else
                //prihlasenie prebehlo pocas poslednej hodiny pocas dna ale nie posledny den v mesiaci
                return new DateTime(interval.Login.Year, interval.Login.Month, interval.Login.Day+1, 0, 0, 0);
        }

        
    }
}
