﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using OccupationMonitorLib.DataObjects;
using OccupationMonitorLib.ProcessingStrategies;

namespace OccupationMonitorLib.Processors
{
    //trieda zodpovedna za spracovanie intervalov prihlasenia
    //konkretny sposob, akym sa budu intervaly spracuvavat reprezentuje atribut procStrategy
    class IntervalsProcessor
    {
        private ProcessingStrategy procStrategy;

        public IntervalsProcessor(ProcessingStrategy strat) 
        {
            procStrategy = strat;
        }

        //spracuje intervaly prihlasenia - detaily v triede ProcessingStrategy a triedach, ktore ju implementuju
        internal int[] ProcessIntervals(List<OccupationInterval> intervals)
        {
            return procStrategy.Process(intervals);
        }

        //pole reprezentujuce pocet sekund, pocas ktorych bol niekto prihlaseny na pocitacoch vyjadri v percentach
        internal float[] ProcessToPercents(int pcsCount, List<OccupationInterval> intervals)
        {
            int[] p = procStrategy.Process(intervals);
            float[] percents = new float[p.Length];

            int maxMinutes = procStrategy.MaxSeconds;
            for (int i = 0; i < percents.Length; i++) 
            {
                percents[i] = (p[i] * 100.0f / (maxMinutes * pcsCount));
            }

            return percents;
        }
    }
}
