﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using OccupationMonitorLib.DataObjects;
using OccupationMonitorLib.DataAccess;
using OccupationMonitorLib.MiniZappa;

namespace OccupationMonitorLib.Processors
{
    
    //trieda zodpovedna za prevod n-arneho stromu na format odpovedajuci zaznamom v databaze
    class MapLocalityProcessor
    {
        LogItemsCatalog itemsCatalog;

        public MapLocalityProcessor() 
        {
            itemsCatalog = new LogItemsCatalog();
        }

        //zaloguje aktualny stav lokality = zaloguje stavy listov (pocitacov)
        internal void LogState(MapLocality locality, Dictionary<long, WorkstationStatus> statuses) 
        {
            long rootId = locality.Id;
            foreach (MapStudyroom studyRoom in locality.Studyrooms) 
            {
                if (studyRoom != null && studyRoom.RootMapObject != null)
                {
                    MapObject rootObject = studyRoom.RootMapObject;
                    LogChildren(rootId, rootObject, statuses);
                }
            }
        }

        //vrati logy z databaze
        internal List<LogItem> GetLogs(long rootId, DateTime from, DateTime to) 
        {
            return itemsCatalog.GetItems(rootId, from, to);
        }

        //zaloguje stavy pocitacov (listov) nachadzajucich sa v prislusnej miestnosti (uzly)
        //vola sa rekurzivne
        private void LogChildren(long rootId, MapObject mapObject, Dictionary<long, WorkstationStatus> statuses)
        {
            foreach (MapObject child in mapObject.Children)
            {
                if (child != null)
                {
                    if (child.ObjectType == MapObjectType.Workstation)
                    {
                        WorkstationStatus status = new WorkstationStatus();
                        //zisti aktualny stav pocitaca
                        if (statuses.ContainsKey((long)child.Data)) statuses.TryGetValue((long)child.Data, out status);
                        else status.StatusValue = WorkstationStatusEnum.NotSet;
                        if (child.Data != null) LogEntry(rootId, (long)child.Data, status.StatusValue);
                    }
                    else if (mapObject.ObjectType == MapObjectType.Studyroom) LogChildren(rootId, child, statuses);//rekurzivne volanie
                }
            }
        }

        private void LogEntry(long rootId, long pcId, WorkstationStatusEnum status) 
        {
            LogItem logItem = itemsCatalog.GetLastLog(rootId, pcId);
            //v pripade, ze aktualny stav pc je iny ako jeho posledny zalogovany stav, tak ho zaloguje
            if (logItem.State != status)
            {
                logItem = new LogItem(rootId, pcId, DateTime.Now, status);
                itemsCatalog.Create(logItem);
            }
        }
        
    }
}
