﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using OccupationMonitorLib.MiniZappa;
using OccupationMonitorLib.DataAccess;

namespace OccupationMonitorLib.TextOutput
{
    class CsvGenerator
    {
        ADMapper _mapper;
 
        public CsvGenerator() 
        {
            _mapper = new ADMapper();
        }

        /// <summary>
        /// vrati retazec vo formate csv, ktory obsahuje DNka pocitacov a ich stavy, v zadanej lokalite
        /// </summary>
        /// <param name="mapLocality"></param>
        /// <param name="statuses"></param>
        /// <returns></returns>
        internal string GetCsvData(MapLocality mapLocality, Dictionary<long, WorkstationStatus> statuses) 
        {
            StringBuilder builder = new StringBuilder();
            Dictionary<long, string> names = _mapper.GetDistinguishNames();
            if (mapLocality == null || mapLocality.Studyrooms == null || statuses == null) 
            {
                return String.Empty;
            }

            for (int i = 0; i < mapLocality.Studyrooms.Length; i++) 
            {
                builder.Append(GetChildrenData(mapLocality.Studyrooms[i].RootMapObject, statuses, names));
            }
        
            return builder.ToString();
        }

        /// <summary>
        /// vrati dvojice id-dn vsetkych pocitacov v danom MapObjecte
        /// </summary>
        /// <param name="room"></param>
        /// <param name="statuses"></param>
        /// <param name="names"></param>
        /// <returns></returns>
        private string GetChildrenData(MapObject room, Dictionary<long, WorkstationStatus> statuses, Dictionary<long, string> names)
        {
            StringBuilder builder = new StringBuilder();
            string dn;
            long pcId;

            foreach (MapObject child in room.Children) 
            {
                if (child.ObjectType == MapObjectType.Workstation)//prida riadok k vysledku
                {
                    pcId = Convert.ToInt64(child.Data);
                    dn = names[pcId];

                    if (statuses.ContainsKey(pcId))//ak slovnik neobsahuje id, stav pc nie je znamy (moze byt vypnuty)
                    {
                        builder.AppendLine(String.Format("{0};{1}", dn, statuses[pcId].StatusValue));
                    }
                    else 
                    {
                        builder.AppendLine(String.Format("{0};{1}", dn, WorkstationStatusEnum.NotSet));
                    }
                }
                else //rekurzivne zanorenie do "podmiestnosti"
                {
                    builder.Append(GetChildrenData(child, statuses, names));
                }
            }
            return builder.ToString();
        }


    }
}
